package org.eclipse.jetty.hazelcast.session;

import com.hazelcast.config.Config;
import com.hazelcast.config.MapConfig;
import com.hazelcast.config.XmlConfigBuilder;
import com.hazelcast.core.Hazelcast;
import com.hazelcast.core.HazelcastInstance;
import org.eclipse.jetty.server.session.AbstractSessionDataStoreFactory;
import org.eclipse.jetty.server.session.SessionDataStore;
import org.eclipse.jetty.server.session.SessionDataStoreFactory;
import org.eclipse.jetty.server.session.SessionHandler;

import java.io.IOException;

/**
 * Factory to construct {@link HazelcastSessionDataStore}
 */
public class HazelcastSessionDataStoreFactory
    extends AbstractSessionDataStoreFactory
    implements SessionDataStoreFactory
{

    private String hazelcastInstanceName = "JETTY_DISTRIBUTED_SESSION_INSTANCE";

    private String configurationLocation;

    private String mapName = "jetty-distributed-session-map";

    private HazelcastInstance hazelcastInstance;

    private MapConfig mapConfig;


    @Override
    public SessionDataStore getSessionDataStore( SessionHandler handler )
        throws Exception
    {
        HazelcastSessionDataStore hazelcastSessionDataStore = new HazelcastSessionDataStore();

        if ( hazelcastInstance == null )
        {
            try
            {
                Config config;
                if ( configurationLocation == null )
                {
                    config = new Config();
                    // configure a default Map if null
                    if ( mapConfig == null )
                    {
                        mapConfig = new MapConfig();
                        mapConfig.setName( mapName );
                    }
                    else
                    {
                        // otherwise we reuse the name
                        mapName = mapConfig.getName();
                    }
                    config.addMapConfig( mapConfig );
                }
                else
                {
                    config = new XmlConfigBuilder( configurationLocation ).build();
                }
                config.setInstanceName( hazelcastInstanceName );
                hazelcastInstance = Hazelcast.getOrCreateHazelcastInstance( config );
            }
            catch ( IOException e )
            {
                throw new RuntimeException( e.getMessage(), e );
            }
        }
        // initialize the map
        hazelcastSessionDataStore.setSessionDataMap(hazelcastInstance.getMap( mapName ) );
        hazelcastSessionDataStore.setGracePeriodSec( getGracePeriodSec() );
        hazelcastSessionDataStore.setSavePeriodSec( getSavePeriodSec() );
        return hazelcastSessionDataStore;
    }

    public String getConfigurationLocation()
    {
        return configurationLocation;
    }

    public void setConfigurationLocation( String configurationLocation )
    {
        this.configurationLocation = configurationLocation;
    }

    public String getMapName()
    {
        return mapName;
    }

    public void setMapName( String mapName )
    {
        this.mapName = mapName;
    }

    public HazelcastInstance getHazelcastInstance()
    {
        return hazelcastInstance;
    }

    public void setHazelcastInstance( HazelcastInstance hazelcastInstance )
    {
        this.hazelcastInstance = hazelcastInstance;
    }

    public MapConfig getMapConfig()
    {
        return mapConfig;
    }

    public void setMapConfig( MapConfig mapConfig )
    {
        this.mapConfig = mapConfig;
    }

    public String getHazelcastInstanceName()
    {
        return hazelcastInstanceName;
    }

    public void setHazelcastInstanceName( String hazelcastInstanceName )
    {
        this.hazelcastInstanceName = hazelcastInstanceName;
    }
}